/*
 * Copyright (c) 2005, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: InterleavedExpression.java 2309 2006-01-26 21:35:52Z jmettraux $
 */

//
// InterleavedExpression.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 20.11.2001 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.engine.expressions;

//import openwfe.org.engine.Definitions;
import openwfe.org.engine.workitem.InFlowWorkItem;


/**
 * An expression directly implementing the "interleaved parallel routing"
 * pattern of the workflow patterns (http://www.workflowpatterns.com) (pat #17).
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-01-26 22:35:52 +0100 (Thu, 26 Jan 2006) $
 * <br>$Id: InterleavedExpression.java 2309 2006-01-26 21:35:52Z jmettraux $ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public class InterleavedExpression

    extends CleanCompositeFlowExpression

{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(InterleavedExpression.class.getName());

    //
    // FIELDS

    //
    // CONSTRUCTORS

    //
    // BEAN METHODS

    //
    // METHODS

    public void apply (final InFlowWorkItem wi) 
        throws ApplyException
    {
        if (getChildren().size() < 1)
        {
            applyToParent(wi);
            return;
        }

        final double d = Math.random();
        final int index = (int)(d * (double)getChildren().size());

        log.debug("apply() d is "+d+"  index is "+index);

        final FlowExpressionId nextId = 
            (FlowExpressionId)getChildren().get(index);

        getExpressionPool().apply(nextId, wi);
    }

    public void reply (final InFlowWorkItem wi) 
        throws ReplyException
    {
        final FlowExpressionId replyingId = wi.getLastExpressionId();

        log.debug("reply() from  "+replyingId);

        getChildren().remove(replyingId);

        this.storeItself();

        try
        {
            apply(wi);
        }
        catch (final ApplyException e)
        {
            throw new ReplyException
                ("failed to apply next (random) child", e);
        }
    }

}
